Attribute VB_Name = "DatabaseSM"
'  FILE:       DatabaseSM.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Contains all the database initialization/cleanup
'              procedures for the project. Also contains upgrade
'              database upgrade functions.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
' This module is called DatabaseSM, since Database is a standard
' Visual Basic object and we want to avoid any confusion with it.

Option Explicit

Public wrkJet As Workspace
Public dbsAttTool As Database
Public gblnDbOpen As Boolean
Public gRunEngine As rdoEngine

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "DatabaseSM."
Public Const gsDefDBFileExt As String = ".stp"
Private Const msDefDBFile As String = "\SMData" & gsDefDBFileExt

Private Const merrFileNotFound As Integer = 3024
Private Const merrDaoTableMissing As Integer = 3078

Private Const STEPMASTER_SETTINGS_VAL_NAME_DBFILE As String = "WorkspaceFile"

Public Const DEF_NO_COUNT_DISPLAY          As Boolean = False
Public Const DEF_NO_EXECUTE                As Boolean = False
Public Const DEF_PARSE_QUERY_ONLY          As Boolean = False
Public Const DEF_ANSI_QUOTED_IDENTIFIERS   As Boolean = False
Public Const DEF_ANSI_NULLS                As Boolean = True
Public Const DEF_SHOW_QUERY_PLAN           As Boolean = False
Public Const DEF_SHOW_STATS_TIME           As Boolean = False
Public Const DEF_SHOW_STATS_IO             As Boolean = False
Public Const DEF_PARSE_ODBC_MSG_PREFIXES   As Boolean = True
Public Const DEF_ROW_COUNT                 As Long = 0
Public Const DEF_TSQL_BATCH_SEPARATOR      As String = "GO"
Public Const DEF_QUERY_TIME_OUT            As Long = 0
Public Const DEF_SERVER_LANGUAGE           As String = "(Default)"
Public Const DEF_CHARACTER_TRANSLATION     As Boolean = True
Public Const DEF_REGIONAL_SETTINGS         As Boolean = False

Public Const PARAM_DEFAULT_DIR             As String = "DEFAULT_DIR"
Public Const PARAM_DEFAULT_DIR_DESC        As String = "Default destination directory " & _
                "for all output and error files. If it is blank, the StepMaster installation directory will be used."

Public Const PARAM_RUN_ID                  As String = "RUN_ID"
Public Const PARAM_RUN_ID_DESC             As String = "The run identifier for a run. " & _
                "Any modifications will be overwritten before each run."

Public Const PARAM_OUTPUT_DIR              As String = "OUTPUT_DIR"
Public Const PARAM_OUTPUT_DIR_DESC         As String = "The output directory for a run. " & _
                "Any modifications will be overwritten before each run."

Public Const CONNECTION_STRINGS_TO_NAME_SUFFIX As String = "_NAME"
                                                        
Private Const TBL_RUN_STEP_HDR As String = "run_header"
Private Const TBL_RUN_STEP_DTLS As String = "run_step_details"
Public Const TBL_CONNECTION_DTLS As String = "connection_dtls"
Public Const TBL_CONNECTION_STRINGS As String = "workspace_connections"
Public Const TBL_STEPS As String = "att_steps"

Public Const FLD_ID_CONN_NAME As String = "connection_name_id"
Public Const FLD_ID_WORKSPACE As String = "workspace_id"
Public Const FLD_ID_STEP As String = "step_id"
Public Const FLD_ID_PARAMETER As String = "parameter_id"

Public Const FLD_CONN_DTL_CONNECTION_NAME As String = "connection_name"
Public Const FLD_CONN_DTL_CONNECTION_STRING As String = "connection_string_name"
Public Const FLD_CONN_DTL_CONNECTION_TYPE As String = "connection_type"

Public Const FLD_CONN_STR_CONNECTION_NAME As String = "connection_name"

Public Const FLD_STEPS_EXEC_MECHANISM As String = "execution_mechanism"
Public Const FLD_STEPS_EXEC_DTL As String = "start_directory"
Public Const FLD_STEPS_VERSION_NO As String = "version_no"

Public Const DATA_TYPE_CURRENCY As String = "CURRENCY"
Public Const DATA_TYPE_LONG As String = "Long"
Public Const DATA_TYPE_INTEGER As String = "INTEGER"
Public Const DATA_TYPE_TEXT255 As String = "Text(255)"

Private Sub InsertBuiltInParameter(dbFile As Database, sParamName As String, _
        sParamValue As String, sParamDesc As String)

    Dim sBuf As String
    Dim cTempStr As New cStringSM
    Dim lId As Long
    Dim rTemp As DAO.Recordset
    Dim rParam As DAO.Recordset
    Dim cTempSeq As cSequence

    ' Create the passed in built-in parameter, for each workspace in the db
    Set cTempSeq = New cSequence
    Set cTempSeq.IdDatabase = dbFile
    cTempSeq.IdentifierColumn = FLD_ID_PARAMETER
    
    sBuf = "select * from att_workspaces "
    Set rTemp = dbFile.OpenRecordset(sBuf, dbOpenSnapshot)
    If rTemp.RecordCount <> 0 Then
        rTemp.MoveFirst
        
        While Not rTemp.EOF
            sBuf = "select * from workspace_parameters " & _
                    " where workspace_id = " & Str(rTemp!workspace_id) & _
                    " and parameter_name = " & cTempStr.MakeStringFieldValid(sParamName)
            Set rParam = dbFile.OpenRecordset(sBuf, dbOpenSnapshot)
            If rParam.RecordCount <> 0 Then
                rParam.MoveFirst
                ' Since the parameter already exists, change it to a built-in type
                sBuf = "update workspace_parameters " & _
                        " set parameter_type = " & CStr(gintParameterBuiltIn) & _
                        ", description = " & cTempStr.MakeStringFieldValid(sParamDesc) & _
                        " where workspace_id = " & Str(rTemp!workspace_id) & _
                        " and parameter_id = " & Str(rParam!parameter_id)
            Else
                ' Else, insert a parameter record
                lId = cTempSeq.Identifier
                sBuf = "insert into workspace_parameters " & _
                        "( workspace_id, parameter_id, " & _
                        " parameter_name, parameter_value, " & _
                        " description, parameter_type ) " & _
                        " values ( " & _
                        Str(rTemp!workspace_id) & ", " & Str(lId) & ", " & _
                        cTempStr.MakeStringFieldValid(sParamName) & ", " & _
                        cTempStr.MakeStringFieldValid(sParamValue) & ", " & _
                        cTempStr.MakeStringFieldValid(sParamDesc) & ", " & _
                        CStr(gintParameterBuiltIn) & _
                        " ) "
            End If
            dbFile.Execute sBuf, dbFailOnError
            rParam.Close
            
            rTemp.MoveNext
        Wend
    End If
    rTemp.Close
    
End Sub
Public Sub InitRunEngine()

    Set gRunEngine = New rdoEngine
    gRunEngine.rdoDefaultCursorDriver = rdUseServer
    
End Sub

Public Function DefaultDBFile() As String
    DefaultDBFile = GetSetting(App.Title, "Settings", STEPMASTER_SETTINGS_VAL_NAME_DBFILE, App.Path & msDefDBFile)
End Function

Public Sub CloseDatabase()
    
    Dim dbsInstance As Database
    Dim recInstance As Recordset
    
    On Error GoTo CloseDatabaseErr

    ' Close all open recordsets and databases in the workspace
    For Each dbsInstance In wrkJet.Databases
    
        For Each recInstance In dbsAttTool.Recordsets
            recInstance.Close
        Next recInstance
        dbsInstance.Close
        
    Next dbsInstance
    
    Set dbsAttTool = Nothing
    
    gblnDbOpen = False
    wrkJet.Close
    
    Exit Sub

CloseDatabaseErr:
    
    Call LogErrors(Errors)
    Resume Next

End Sub

Private Function NoDbChanges(sVerTo As String, sVerFrom As String) As Boolean

    If sVerTo = gsVersion242 And sVerFrom = gsVersion241 Then
        NoDbChanges = True
    ElseIf sVerTo = gsVersion242 And sVerFrom = gsVersion24 Then
        NoDbChanges = True
    ElseIf sVerTo = gsVersion253 And sVerFrom = gsVersion251 Then
        NoDbChanges = True
    ElseIf sVerTo = gsVersion255 And sVerFrom = gsVersion251 Then
        NoDbChanges = True
    ElseIf sVerTo = gsVersion270 And sVerFrom = gsVersion251 Then
        NoDbChanges = True
    Else
        NoDbChanges = False
    End If
    
End Function

Public Function SMOpenDatabase(Optional strDbName As String = gstrEmptyString) As Boolean
    Dim sVersion As String
    Dim bOpeningDb As Boolean ' This flag is used to check if OpenDatabase failed

    On Error GoTo OpenDatabaseErr
    
    bOpeningDb = False
    SMOpenDatabase = False

    ' Create Microsoft Jet Workspace object.
    If Not gblnDbOpen Then
        Set wrkJet = CreateWorkspace("att_tool_workspace_setup", "admin", gstrEmptyString, dbUseJet)
    End If

    ' Prompt the user for the database file if it is not passed in
    If StringEmpty(strDbName) Then
        strDbName = BrowseDBFile
        If StringEmpty(strDbName) Then
            Exit Function
        End If
    End If

    Do
        If gblnDbOpen Then
#If Not RUN_ONLY Then
            CloseOpenWorkspaces
#End If
            Set wrkJet = CreateWorkspace("att_tool_workspace_setup", "admin", gstrEmptyString, dbUseJet)
        End If

        ' Toggle the bOpeningDb flag around the OpenDatabase method - the value
        ' of this flag will be checked by the error handler to determine if it is
        ' the OpenDatabase that failed.
        BugMessage "DB File: " & strDbName
        
        bOpeningDb = True
        ' Open the database for exclusive use
        Set dbsAttTool = wrkJet.OpenDatabase(strDbName, Options:=True)
        bOpeningDb = False
    
        If dbsAttTool Is Nothing Then
            ' If the file is not present in the directory, display
            ' an error and ask the user to enter a new path
            Call ShowError(errOpenDbFailed, OptArgs:=strDbName)
            
            strDbName = BrowseDBFile
        Else
            sVersion = DBVersion(dbsAttTool)
            
            ' Make sure the application and db version numbers match
            If sVersion = gsVersion Then
                Call InitializeData(strDbName)
                gblnDbOpen = True
                SMOpenDatabase = True
            Else
                If UpgradeDb(wrkJet, dbsAttTool, gsVersion, sVersion) Then
                    Call InitializeData(strDbName)
                    gblnDbOpen = True
                    SMOpenDatabase = True
                Else
                    dbsAttTool.Close
                    Set dbsAttTool = Nothing
    
                    ShowError errVersionMismatch, _
                            OptArgs:=" Please install Version '" & gsVersion & "' of the workspace definition file."
                    strDbName = BrowseDBFile
                End If
            End If
        End If
    Loop While gblnDbOpen = False And Not StringEmpty(strDbName)
    
    Exit Function

OpenDatabaseErr:
    Call DisplayErrors(Errors)
    
    ' If the OpenDatabase failed, continue
    If bOpeningDb Then
        Resume Next
    End If
    
    Call ShowError(errOpenDbFailed, OptArgs:=strDbName)

End Function
Private Sub InitializeData(sDb As String)

    Set gcParameters = New cArrParameters
    Set gcParameters.ParamDatabase = dbsAttTool
    
    Set gcSteps = New cArrSteps
    Set gcSteps.StepDB = dbsAttTool
    
    Set gcConstraints = New cArrConstraints
    Set gcConstraints.ConstraintDB = dbsAttTool

    Set gcConnections = New cConnections
    Set gcConnections.ConnDb = dbsAttTool
    
    Set gcConnDtls = New cConnDtls
    Set gcConnDtls.ConnDb = dbsAttTool
    
    ' Disable the error handler since this is not a critical step
    On Error GoTo 0
    SaveSetting App.Title, "Settings", STEPMASTER_SETTINGS_VAL_NAME_DBFILE, sDb
End Sub
Private Sub UpdateContinuationCriteria(dbFile As DAO.Database)
    
    Dim qyTemp As DAO.QueryDef
    Dim sBuf As String

    On Error GoTo UpdateContinuationCriteriaErr
    
    sBuf = "Since this version of the executable incorporates failure processing, " & _
            "the upgrade will update the On Failure field for each of the steps " & _
            "to 'Continue' to be compatible with the existing behaviour. " & _
            "Proceed?"
    If Not Confirm(Buttons:=vbYesNo, strMessage:=sBuf, strTitle:="Upgrade database") Then
        Exit Sub
    End If
    
    ' Create a recordset object to retrieve all steps for
    ' the given workspace
    sBuf = " update att_steps a " & _
        " set continuation_criteria = " & CStr(gintOnFailureContinue) & _
        " where archived_flag = [archived] "

    ' Find the highest X-component of the version number
    sBuf = sBuf & " AND cint( mid( version_no, 1, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) = " & _
        " ( select max( cint( mid( version_no, 1, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) ) " & _
        " from att_steps AS d " & _
        " WHERE a.step_id = d.step_id ) "
        
    ' Find the highest Y-component of the version number for the highest X-component
    sBuf = sBuf & " AND cint( mid( version_no, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) + 1 ) ) = " & _
        " ( select max( cint( mid( version_no, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) + 1 ) ) ) " & _
        " from att_steps AS b " & _
        " Where a.step_id = b.step_id " & _
        " AND cint( mid( version_no, 1, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) = " & _
        " ( select max( cint( mid( version_no, 1, instr( version_no, " & gstrDQ & gstrVerSeparator & gstrDQ & " ) - 1 ) ) ) " & _
        " from att_steps AS c " & _
        " WHERE a.step_id = c.step_id ) ) "
    
    ' Create a temporary Querydef object
    Set qyTemp = dbFile.CreateQueryDef(gstrEmptyString, sBuf)
    qyTemp.Parameters("archived").Value = False
    
    qyTemp.Execute dbFailOnError
    qyTemp.Close
    
    Exit Sub
    
UpdateContinuationCriteriaErr:
    Call LogErrors(Errors)
    Err.Raise vbObjectError + errModifyStepFailed, mstrModuleName, _
            LoadResString(errModifyStepFailed)
    
End Sub

Private Sub UpdateDbDtls(dbFile As Database, sNewVersion As String)

    Dim sSql As String
    Dim cTemp As New cStringSM

    On Error GoTo UpdateDbDtlsErr
    
    sSql = "update db_details " & _
            " set db_version = " & cTemp.MakeStringFieldValid(sNewVersion)

    dbFile.Execute sSql, dbFailOnError
    
    Exit Sub

UpdateDbDtlsErr:
    Call LogErrors(Errors)
    Err.Raise vbObjectError + errUpgradeFailed, mstrModuleName, _
            LoadResString(errUpgradeFailed)
    
End Sub

Private Sub Upgrade10to21(UpgradeWsp As DAO.Workspace, dbFile As Database, sVersion As String)

    Dim sSql As String

    On Error GoTo Upgrade10to21Err
    
    Call UpdateDbDtls(dbFile, sVersion)
    
    Call UpdateContinuationCriteria(dbFile)
    
    Exit Sub

Upgrade10to21Err:
    UpgradeWsp.Rollback
    Call LogErrors(Errors)
    Err.Raise vbObjectError + errUpgradeFailed, mstrModuleName, _
            LoadResString(errUpgradeFailed)
    
End Sub
Private Sub Upgrade21to23(UpgradeWsp As DAO.Workspace, dbFile As Database, sVersion As String)

    Dim sBuf As String
    Dim cTempStr As New cStringSM

    On Error GoTo Upgrade21to23Err
    
    ' Add a parameter type field and a description field to the parameter table
    sBuf = "alter table workspace_parameters " & _
            " add column description TEXT(255) "
    dbFile.Execute sBuf, dbFailOnError

    sBuf = "alter table workspace_parameters " & _
            " add column parameter_type INTEGER "
    dbFile.Execute sBuf, dbFailOnError
    
    ' Initialize the parameter type on all parameters to indicate generic parameters
    sBuf = "update workspace_parameters " & _
            " set parameter_type = " & CStr(gintParameterGeneric)
    dbFile.Execute sBuf, dbFailOnError

    sBuf = "Release 2.3 onwards, connection string parameters will be " & _
            "displayed in a separate node. After this upgrade, all connection " & _
            "string parameters will appear under the Globals/Connection Strings " & _
            "node in the workspace. "
    Call MsgBox(sBuf, vbOKOnly + vbApplicationModal, "Upgrade database")
            
    ' Update the parameter type on all parameters that look like db connection strings
    sBuf = "update workspace_parameters " & _
            " set parameter_type = " & CStr(gintParameterConnect) & _
            " where UCase(parameter_value) like '*DRIVER*' " & _
            " or UCase(parameter_value) like '*DSN*'"
    dbFile.Execute sBuf, dbFailOnError

    ' Add an elapsed time field to the run_step_details table - this field is
    ' needed to store the elapsed time in milliseconds.
    sBuf = "alter table run_step_details " & _
            " add column elapsed_time LONG "
    dbFile.Execute sBuf, dbFailOnError
    
    ' The failure_details field has some data for the case when an ODBC failure
    ' threshold was specified. Since that's no longer relevant, update the failure_details
    ' field for records with failure_criteria = gintFailureODBC to empty.
    ' failure_criteria = gintFailureODBC = 1
    sBuf = "update att_steps " & _
            " set failure_details = " & cTempStr.MakeStringFieldValid(gstrEmptyString) & _
            " where failure_criteria = '1'"
    dbFile.Execute sBuf, dbFailOnError
    
    Call UpdateDbDtls(dbFile, sVersion)
    
    UpgradeWsp.CommitTrans
    
    On Error GoTo DropColumnErr
    
    UpgradeWsp.BeginTrans
    
    ' This ddl cannot be in the same transaction as the failure_details update
    ' But we can do this in a separate transaction since we do not expect this
    ' statement to fail - AND, it doesn't matter if this transaction fails
    ' Drop the failure_criteria column from the att_steps table
    sBuf = "alter table att_steps " & _
            " drop column failure_criteria "
    dbFile.Execute sBuf, dbFailOnError
    
    Exit Sub
    
DropColumnErr:
    Call LogErrors(Errors)
    ShowError errDeleteColumnFailed
    Exit Sub

Upgrade21to23Err:
    UpgradeWsp.Rollback
    Call LogErrors(Errors)
    Err.Raise vbObjectError + errUpgradeFailed, mstrModuleName, _
            LoadResString(errUpgradeFailed)
    
End Sub
Private Sub Upgrade23to24(UpgradeWsp As DAO.Workspace, dbFile As Database, sVersion As String)

    Dim sBuf As String
    Dim lId As Long
    Dim rTemp As DAO.Recordset
    Dim cTempStr As New cStringSM

    On Error GoTo Upgrade23to24Err
    
    ' Add a new table for connection properties
    sBuf = CreateConnectionsTableScript()
    ' TODO: Not sure of column sizes for row count, tsql_batch_separator and server_language
    dbFile.Execute sBuf, dbFailOnError
    
    ' Move all connection parameters from the parameter table to the connections tables
    ' Insert default values for the newly added connection properties
    sBuf = "select * from workspace_parameters " & _
             "where parameter_type = " & CStr(gintParameterConnect)
    Set rTemp = dbFile.OpenRecordset(sBuf, dbOpenSnapshot)
    lId = 1
    If rTemp.RecordCount <> 0 Then
        rTemp.MoveFirst
        
        While Not rTemp.EOF
            sBuf = "insert into workspace_connections " & _
             "( workspace_id, connection_id, " & _
             "connection_name, connection_value, " & _
             "description, no_count_display, " & _
             "no_execute, parse_query_only, " & _
             "ANSI_quoted_identifiers, ANSI_nulls, " & _
             "show_query_plan, show_stats_time, " & _
             "show_stats_io, parse_odbc_msg_prefixes, " & _
             "row_count, tsql_batch_separator, " & _
             "query_time_out, server_language, " & _
             "character_translation, regional_settings ) " & _
            " values ( " & _
            Str(rTemp!workspace_id) & ", " & Str(lId) & ", " & _
            cTempStr.MakeStringFieldValid("" & rTemp!parameter_name) & ", " & _
            cTempStr.MakeStringFieldValid("" & rTemp!parameter_value) & ", " & _
            cTempStr.MakeStringFieldValid("" & rTemp!Description) & ", " & _
            Str(DEF_NO_COUNT_DISPLAY) & ", " & _
            Str(DEF_NO_EXECUTE) & ", " & Str(DEF_PARSE_QUERY_ONLY) & ", " & _
            Str(DEF_ANSI_QUOTED_IDENTIFIERS) & ", " & Str(DEF_ANSI_NULLS) & ", " & _
            Str(DEF_SHOW_QUERY_PLAN) & ", " & Str(DEF_SHOW_STATS_TIME) & ", " & _
            Str(DEF_SHOW_STATS_IO) & ", " & Str(DEF_PARSE_ODBC_MSG_PREFIXES) & ", " & _
            Str(DEF_ROW_COUNT) & ", " & cTempStr.MakeStringFieldValid(DEF_TSQL_BATCH_SEPARATOR) & ", " & _
            Str(DEF_QUERY_TIME_OUT) & ", " & cTempStr.MakeStringFieldValid(DEF_SERVER_LANGUAGE) & ", " & _
            Str(DEF_CHARACTER_TRANSLATION) & ", " & Str(DEF_REGIONAL_SETTINGS) & _
            " ) "
            dbFile.Execute sBuf, dbFailOnError
            
            lId = lId + 1
            rTemp.MoveNext
        Wend
    End If
    rTemp.Close
    
    ' Add an identifier column for the connection_id field
    sBuf = "alter table att_identifiers " & _
            " add column connection_id long "
    dbFile.Execute sBuf, dbFailOnError

    ' Initialize the value of the connection identifier, initialized above
    sBuf = "update att_identifiers " & _
            " set connection_id = " & Str(lId)
    dbFile.Execute sBuf, dbFailOnError

    ' Delete all connection strings from the parameter table
    sBuf = "delete from workspace_parameters " & _
             "where parameter_type = " & CStr(gintParameterConnect)
    dbFile.Execute sBuf, dbFailOnError

    ' Create the built-in parameter, default directory, for each workspace in the db
    Call InsertBuiltInParameter(dbFile, PARAM_DEFAULT_DIR, gstrEmptyString, PARAM_DEFAULT_DIR_DESC)
    
    Call UpdateDbDtls(dbFile, sVersion)
    
    Exit Sub

Upgrade23to24Err:
    UpgradeWsp.Rollback
    Call LogErrors(Errors)
    Err.Raise vbObjectError + errUpgradeFailed, mstrModuleName, _
            LoadResString(errUpgradeFailed)
    
End Sub
Private Sub Upgrade243to25(UpgradeWsp As DAO.Workspace, dbFile As Database, sVersion As String)

    Dim sBuf As String
    Dim qy As DAO.QueryDef
    Dim rTemp As DAO.Recordset
    Dim lId As Long
    Dim cTempStr As New cStringSM

    On Error GoTo Upgrade243to25Err
    
    sBuf = "Release " & gsVersion25 & " onwards, new 'Connections' must be created for all " & _
            "connection strings. " & vbCrLf & vbCrLf & _
            "Connections will appear under the Globals/Connections " & _
            "node in the workspace. " & vbCrLf & _
            "A list of all 'Connections' (instead of 'Connection Strings') " & _
            "in the workspace will be displayed in the 'Connections' field for " & _
            "ODBC steps on the Step definition screen. " & vbCrLf & vbCrLf & _
            "Each Connection can be marked as static or dynamic. " & vbCrLf & _
            "Dynamic connections will be created when a step starts execution and " & _
            "closed once the step completes. " & vbCrLf & _
            "Static connections will be kept open till the run completes." & vbCrLf & vbCrLf & _
            "Currently dynamic 'Connections' have been created for all existing 'Connection Strings' " & _
            "with the suffix " & CONNECTION_STRINGS_TO_NAME_SUFFIX
    Call MsgBox(sBuf, vbOKOnly + vbApplicationModal, "Upgrade database")
    
    ' Add a new table for the connection name entity
    ' This table has been added in order to satisfy the TPC-H requirement that
    ' all the queries in a stream need to be executed on a single connection.
    sBuf = CreateConnectionDtlsTableScript()
    dbFile.Execute sBuf, dbFailOnError
    
    ' Add an identifier column for the connection_name_id field
    sBuf = "alter table att_identifiers " & _
            " add column " & FLD_ID_CONN_NAME & " long "
    dbFile.Execute sBuf, dbFailOnError

    Call UpdateDbDtls(dbFile, sVersion)
    
    ' insert connection_dtl records for each of the connection strings
    sBuf = "select * from " & TBL_CONNECTION_STRINGS
    Set rTemp = dbFile.OpenRecordset(sBuf, dbOpenSnapshot)
    
    sBuf = "insert into " & TBL_CONNECTION_DTLS & _
             "( " & FLD_ID_WORKSPACE & _
             ", " & FLD_ID_CONN_NAME & _
             ", " & FLD_CONN_DTL_CONNECTION_NAME & _
             ", " & FLD_CONN_DTL_CONNECTION_STRING & _
             ", " & FLD_CONN_DTL_CONNECTION_TYPE & " ) " & _
            " values ( [w_id], [c_id], [c_name], [c_str], [c_type] ) "
    Set qy = dbFile.CreateQueryDef("", sBuf)
    
    lId = glMinId
    If rTemp.RecordCount <> 0 Then
        rTemp.MoveFirst
        
        While Not rTemp.EOF
            qy.Parameters("w_id").Value = rTemp.Fields(FLD_ID_WORKSPACE)
            qy.Parameters("c_id").Value = lId
            qy.Parameters("c_name").Value = rTemp.Fields(FLD_CONN_STR_CONNECTION_NAME) & CONNECTION_STRINGS_TO_NAME_SUFFIX
            qy.Parameters("c_str").Value = rTemp.Fields(FLD_CONN_STR_CONNECTION_NAME)
            qy.Parameters("c_type").Value = ConnTypeDynamic
            
            qy.Execute dbFailOnError
            
            lId = lId + 1
            rTemp.MoveNext
        Wend
    End If
    qy.Close
    rTemp.Close
    
    ' Initialize the value of the connection_name_id
    sBuf = "update att_identifiers " & _
            " set " & FLD_ID_CONN_NAME & " = " & Str(lId)
    dbFile.Execute sBuf, dbFailOnError

    ' Update the start_directory field in att_steps to point to the newly
    ' created connections
    Call ReadStepsInWorkspace(rTemp, qy, glInvalidId, dbLoad:=dbFile, _
            bSelectArchivedRecords:=False)
    
    sBuf = "update " & TBL_STEPS & _
        " set " & FLD_STEPS_EXEC_DTL & " = [c_name] " & _
        " where " & FLD_ID_STEP & " = [s_id] " & _
        " and " & FLD_STEPS_VERSION_NO & " = [ver_no] "
    Set qy = dbFile.CreateQueryDef("", sBuf)

    If rTemp.RecordCount <> 0 Then
        rTemp.MoveFirst
        
        While Not rTemp.EOF
            If rTemp.Fields(FLD_STEPS_EXEC_MECHANISM).Value = gintExecuteODBC Then
                If Not (StringEmpty("" & rTemp.Fields(FLD_STEPS_EXEC_DTL))) Then
                    sBuf = rTemp.Fields(FLD_STEPS_EXEC_DTL)
                    ' Strip the enclosing "%" characters
                    sBuf = Mid(sBuf, 2, Len(sBuf) - 2) & CONNECTION_STRINGS_TO_NAME_SUFFIX
                    
                    qy.Parameters("c_name").Value = sBuf
                    qy.Parameters("s_id").Value = rTemp.Fields(FLD_ID_STEP)
                    qy.Parameters("ver_no").Value = rTemp.Fields(FLD_STEPS_VERSION_NO)
                    
                    qy.Execute dbFailOnError
                End If
            End If
            rTemp.MoveNext
        Wend
    End If
    
    qy.Close
    rTemp.Close
    
    Exit Sub

Upgrade243to25Err:
    UpgradeWsp.Rollback
    Call LogErrors(Errors)
    Err.Raise vbObjectError + errUpgradeFailed, mstrModuleName, _
            LoadResString(errUpgradeFailed)
    
End Sub
Private Sub Upgrade25to251(UpgradeWsp As DAO.Workspace, dbFile As Database, sVersion As String)

    On Error GoTo Upgrade25to251Err
    
    ' Create the built-in parameters, run_id and output_dir, for each workspace in the db
    Call InsertBuiltInParameter(dbFile, PARAM_RUN_ID, gstrEmptyString, PARAM_RUN_ID_DESC)
    Call InsertBuiltInParameter(dbFile, PARAM_OUTPUT_DIR, gstrEmptyString, PARAM_OUTPUT_DIR_DESC)
    
    Call UpdateDbDtls(dbFile, sVersion)
    
    Exit Sub

Upgrade25to251Err:
    UpgradeWsp.Rollback
    Call LogErrors(Errors)
    Err.Raise vbObjectError + errUpgradeFailed, mstrModuleName, _
            LoadResString(errUpgradeFailed)
    
End Sub

Private Sub Upgrade242to243(UpgradeWsp As DAO.Workspace, dbFile As Database, sVersion As String)

    Dim sBuf As String
    Dim cTempStr As New cStringSM
    Dim iResponse As Integer

    On Error GoTo DeleteHistoryErr
    
    Call DeleteRunHistory(dbFile)
    
    On Error GoTo Upgrade242to243Err
    
    UpgradeWsp.CommitTrans
    
    UpgradeWsp.BeginTrans
    
    ' Add a parameter type field and a description field to the parameter table
    sBuf = "alter table run_step_details " & _
            " add column parent_instance_id LONG "

    dbFile.Execute sBuf, dbFailOnError

    sBuf = "alter table run_step_details " & _
            " add column iterator_value TEXT(255) "

    dbFile.Execute sBuf, dbFailOnError

    Call AlterFieldType(dbFile, TBL_RUN_STEP_DTLS, "start_time", DATA_TYPE_CURRENCY)
    Call AlterFieldType(dbFile, TBL_RUN_STEP_DTLS, "end_time", DATA_TYPE_CURRENCY)
    Call AlterFieldType(dbFile, TBL_RUN_STEP_HDR, "start_time", DATA_TYPE_CURRENCY)
    Call AlterFieldType(dbFile, TBL_RUN_STEP_HDR, "end_time", DATA_TYPE_CURRENCY)
    
    Call UpdateDbDtls(dbFile, sVersion)
    
    Exit Sub
    
DeleteHistoryErr:
    ' This is not a critical error - continue with upgrade
    Call LogErrors(Errors)
    Resume Next

Upgrade242to243Err:
    UpgradeWsp.Rollback
    Call LogErrors(Errors)
    Err.Raise vbObjectError + errUpgradeFailed, mstrModuleName, _
            LoadResString(errUpgradeFailed)
    
End Sub
'*****************************************************************
' The AlterFieldType Sub procedure requires three string
' parameters. The first string specifies the name of the table
' containing the field to be changed. The second string specifies
' the name of the field to be changed. The third string specifies
' the new data type for the field.
'*****************************************************************

Private Sub AlterFieldType(dbFile As Database, TblName As String, FieldName As String, _
                    NewDataType As String)
    Dim qdf As DAO.QueryDef
    Dim sSql As String
    
    ' Add a temporary field to the table.
    sSql = "ALTER TABLE [" & TblName & _
            "] ADD COLUMN AlterTempField " & NewDataType
    Set qdf = dbFile.CreateQueryDef("", sSql)
    qdf.Execute
    
    ' Copy the data from old field into the new field.
    qdf.SQL = "UPDATE DISTINCTROW [" & TblName & "] SET AlterTempField = [" & FieldName & "]"
    qdf.Execute
    
    ' Delete the old field.
    qdf.SQL = "ALTER TABLE [" & TblName & "] DROP COLUMN [" & FieldName & "]"
    qdf.Execute
    
    ' Rename the temporary field to the old field's name.
    dbFile.TableDefs("[" & TblName & "]").Fields("AlterTempField").Name = FieldName
    dbFile.TableDefs.Refresh
    
    ' Clean up.
End Sub
Private Sub Upgrade01to21(UpgradeWsp As DAO.Workspace, dbFile As DAO.Database, sVersion As String)
    Dim sSql As String

    On Error GoTo Upgrade01to21Err
    
    sSql = "Create table db_details (" & _
             "db_version                 Text(50) " & _
    ");"

    dbFile.Execute sSql, dbFailOnError
    
    sSql = "insert into db_details " & _
            "( db_version ) values ( '" & sVersion & "' ) "

    dbFile.Execute sSql, dbFailOnError
    
    Call UpdateContinuationCriteria(dbFile)
    
    Exit Sub

Upgrade01to21Err:
    Call LogErrors(Errors)
    UpgradeWsp.Rollback
    Err.Raise vbObjectError + errUpgradeFailed, mstrModuleName, _
            LoadResString(errUpgradeFailed)

End Sub
Private Function UpgradeDb(UpgradeWsp As DAO.Workspace, dbFile As Database, _
        sVerTo As String, sVerFrom As String) As Boolean
    
    Dim sMsg As String

    On Error GoTo UpgradeDbErr
    
    UpgradeDb = False
    If Not ValidUpgrade(sVerTo, sVerFrom) Then Exit Function
    
    If NoDbChanges(sVerTo, sVerFrom) Then
        UpgradeDb = True
        Exit Function
    End If
    
    sMsg = "The database needs to be upgraded from Version " & sVerFrom & _
            " to Version " & sVerTo & "." & vbCrLf & _
            "Proceed?"
    If Not Confirm(Buttons:=vbYesNo, strMessage:=sMsg, strTitle:="Upgrade database") Then
        Exit Function
    End If
    
    UpgradeWsp.BeginTrans
    
    Select Case sVerFrom
        Case gsVersion25
            Call Upgrade25to251(UpgradeWsp, dbFile, gsVersion251)
            
        Case gsVersion243
            Call Upgrade243to25(UpgradeWsp, dbFile, gsVersion25)
            Call Upgrade25to251(UpgradeWsp, dbFile, gsVersion251)
            
        Case gsVersion24, gsVersion241, gsVersion242
            sMsg = "After this upgrade, the run history for previous runs will no longer be available. " & _
                    "Continue?"
            If Not Confirm(Buttons:=vbYesNo, strMessage:=sMsg, strTitle:="Upgrade database") Then
                UpgradeWsp.CommitTrans
                Exit Function
            End If
            Call Upgrade242to243(UpgradeWsp, dbFile, gsVersion243)
            Call Upgrade243to25(UpgradeWsp, dbFile, gsVersion25)
            Call Upgrade25to251(UpgradeWsp, dbFile, gsVersion251)
            
        Case gsVersion23
            Call Upgrade23to24(UpgradeWsp, dbFile, gsVersion24)
            Call Upgrade242to243(UpgradeWsp, dbFile, gsVersion242)
            Call Upgrade243to25(UpgradeWsp, dbFile, gsVersion25)
            Call Upgrade25to251(UpgradeWsp, dbFile, gsVersion251)
            
        Case gsVersion21
            Call Upgrade21to23(UpgradeWsp, dbFile, gsVersion23)
            Call Upgrade23to24(UpgradeWsp, dbFile, gsVersion24)
            Call Upgrade242to243(UpgradeWsp, dbFile, gsVersion242)
            Call Upgrade243to25(UpgradeWsp, dbFile, gsVersion25)
            Call Upgrade25to251(UpgradeWsp, dbFile, gsVersion251)
            
        Case gsVersion10
            Call Upgrade10to21(UpgradeWsp, dbFile, gsVersion21)
            Call Upgrade21to23(UpgradeWsp, dbFile, gsVersion23)
            Call Upgrade23to24(UpgradeWsp, dbFile, gsVersion24)
            Call Upgrade242to243(UpgradeWsp, dbFile, gsVersion242)
            Call Upgrade243to25(UpgradeWsp, dbFile, gsVersion25)
            Call Upgrade25to251(UpgradeWsp, dbFile, gsVersion251)
            
        Case gsVersion01
            Call Upgrade01to21(UpgradeWsp, dbFile, gsVersion21)
            Call Upgrade21to23(UpgradeWsp, dbFile, gsVersion23)
            Call Upgrade23to24(UpgradeWsp, dbFile, gsVersion24)
            Call Upgrade242to243(UpgradeWsp, dbFile, gsVersion242)
            Call Upgrade243to25(UpgradeWsp, dbFile, gsVersion25)
            Call Upgrade25to251(UpgradeWsp, dbFile, gsVersion251)
            
    End Select
            
    UpgradeWsp.CommitTrans
    
    UpgradeDb = True
    Exit Function

UpgradeDbErr:
    Call LogErrors(Errors)
    ShowError errUpgradeFailed
    
End Function
Private Function DBVersion(TestDb As Database) As String
    ' Retrieves the database version
    Dim rVersion As Recordset

    On Error GoTo DBVersionErr
    
    Set rVersion = TestDb.OpenRecordset("Select db_version from db_details ", _
            dbOpenForwardOnly)
    
    BugAssert rVersion.RecordCount <> 0
    DBVersion = rVersion!db_version
    
    rVersion.Close
    Exit Function
    
DBVersionErr:
    If Err.Number = merrDaoTableMissing Then
        DBVersion = gsVersion01
    Else
        LogErrors Errors
        Err.Raise vbObjectError + errUpgradeFailed, mstrModuleName, _
                LoadResString(errUpgradeFailed)
    End If

End Function

Private Function ValidUpgrade(sVerTo As String, sVerFrom As String) As Boolean

    If sVerTo = gsVersion And sVerFrom = gsVersion251 Then
        ValidUpgrade = True
    ElseIf sVerTo = gsVersion And sVerFrom = gsVersion25 Then
        ValidUpgrade = True
    ElseIf sVerTo = gsVersion And sVerFrom = gsVersion243 Then
        ValidUpgrade = True
    ElseIf sVerTo = gsVersion And sVerFrom = gsVersion242 Then
        ValidUpgrade = True
    ElseIf sVerTo = gsVersion And sVerFrom = gsVersion241 Then
        ValidUpgrade = True
    ElseIf sVerTo = gsVersion And sVerFrom = gsVersion24 Then
        ValidUpgrade = True
    ElseIf sVerTo = gsVersion And sVerFrom = gsVersion23 Then
        ValidUpgrade = True
    ElseIf sVerTo = gsVersion And sVerFrom = gsVersion21 Then
        ValidUpgrade = True
    ElseIf sVerTo = gsVersion And sVerFrom = gsVersion10 Then
        ValidUpgrade = True
    ElseIf sVerTo = gsVersion And sVerFrom = gsVersion01 Then
        ValidUpgrade = True
    Else
        ValidUpgrade = False
    End If
    
End Function

