Attribute VB_Name = "DebugSM"
'  FILE:       DebugSM.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Contains all the functions that carry out error/debug
'              processing for the project.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
' Most of the functions in this module that manipulate the
' error object do not have an On Error GoTo statement - this
' is because it will clear the passed in error object - let
' the calling functions handle the errors raised by this
' module, if any
Option Explicit

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "DebugSM."

Private mcLogFile As cFileSM
Private mcErrorFile As cFileSM

Private Const FORMAT_MESSAGE_FROM_SYSTEM = &H1000
Private Const FORMAT_MESSAGE_IGNORE_INSERTS = &H200
Private Const pNull = 0

Declare Function FormatMessage Lib "kernel32" Alias "FormatMessageA" (ByVal dwFlags As Long, lpSource As Any, ByVal dwMessageId As Long, ByVal dwLanguageId As Long, ByVal lpbuffer As String, ByVal nSize As Long, Arguments As Long) As Long
Public Function Confirm(Optional lngMessageCode As conConfirmMsgCodes, _
        Optional lngTitleCode As conConfirmMsgTitleCodes, _
        Optional TitleParameter As String, _
        Optional ByVal Buttons As Integer = -1, _
        Optional strMessage As String = gstrEmptyString, _
        Optional strTitle As String = gstrEmptyString) _
        As Boolean
    ' Displays a confirmation message corresponding to the
    ' passed in message code. Returns True if the user says
    ' Ok and False otherwise
    
    Dim intResponse As Integer
    Dim intButtonStyle As Integer
        
    On Error GoTo ConfirmErr
    
    Confirm = False
    
    ' If the buttons style hasn't been specified, set the
    ' default style to display OK and Cancel buttons
    If Buttons = -1 Then
        intButtonStyle = vbOKCancel
    Else
        intButtonStyle = Buttons
    End If
    
    ' Find the message string for the passed in code
    If StringEmpty(strMessage) Then
        strMessage = Trim$(LoadResString(lngMessageCode))
    End If
    
    If StringEmpty(strTitle) Then
        strTitle = Trim$(LoadResString(lngTitleCode))
    End If
    
    If Not StringEmpty(TitleParameter) Then
        strTitle = strTitle & Chr$(vbKeySpace) & _
                gstrSQ & TitleParameter & gstrSQ
    End If
    
    ' Display the confirmation message with the Cancel button
    ' set to the default - assume that we are confirming
    ' potentially dangerous operations!
    intResponse = MsgBox(strMessage, _
            intButtonStyle + vbQuestion + vbApplicationModal, _
            strTitle)
            
    ' Translate the user response into a True/False return code
    If intButtonStyle = vbOKCancel Then
        If intResponse = vbOK Then
            Confirm = True
        Else
            Confirm = False
        End If
    Else
        If intResponse = vbYes Then
            Confirm = True
        Else
            Confirm = False
        End If
    End If
    
    Exit Function
    
ConfirmErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "Confirm"
    Err.Raise vbObjectError + errConfirmFailed, _
            gstrSource, _
            LoadResString(errConfirmFailed)

End Function
Public Sub LogSystemError()
    Dim eErrCode As Long
    
    eErrCode = GetLastError()
    If eErrCode <> 0 Then
        WriteToFile "System Error: " & eErrCode & vbCrLf & ApiError(eErrCode), _
                blnError:=True
    End If
    
End Sub
Public Function ApiError(ByVal e As Long) As String

    Dim s As String
    Dim c As Long
    
    s = String(256, 0)
    c = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM Or _
            FORMAT_MESSAGE_IGNORE_INSERTS, _
            pNull, e, 0&, s, Len(s), ByVal pNull)
    If c Then ApiError = e & ": " & Left$(s, c)
    
End Function

' Output flags determine output destination of BugAsserts and messages
#Const afLogfile = 1
#Const afMsgBox = 2
#Const afDebugWin = 4
#Const afAppLog = 8

' Display appropriate error message, and then stop
' program.  These errors should NOT be possible in
' shipping product.
Sub BugAssert(ByVal fExpression As Boolean, _
              Optional sExpression As String)
#If afDebug Then
    If fExpression Then Exit Sub
    BugMessage "BugAssert failed: " & sExpression
    Stop
#End If
End Sub
    
Sub BugMessage(sMsg As String)

#If afDebug And afLogfile Then
    ' Since we are writing log messages, the error flag is turned off
    Call WriteToFile(sMsg, False)
#End If
#If afDebug And afMsgBox Then
    MsgBox sMsg
#End If
#If afDebug And afDebugWin Then
    Debug.Print sMsg
#End If
#If afDebug And afAppLog Then
    App.LogEvent sMsg
#End If

End Sub
Public Function ProjectLogFile() As String

    ProjectLogFile = mcLogFile.FileName
    
End Function
Public Function ProjectErrorFile() As String

    ProjectErrorFile = mcErrorFile.FileName
    
End Function

Private Sub WriteToFile(sMsg As String, Optional ByVal blnError As Boolean)

    ' Calls procedures to write the passed in message to the log -
    ' The blnError flag is used to indicate that the message
    ' should be logged to the error file - by default the log
    ' file is used
    
    Dim mcFileObj As cFileSM
    Dim strFileName As String
    Dim strFileHdr As String
    
    On Error GoTo WriteToFileErr

    If blnError Then
        If mcErrorFile Is Nothing Then
            Set mcErrorFile = New cFileSM
        End If
        Set mcFileObj = mcErrorFile
    Else
        If mcLogFile Is Nothing Then
            Set mcLogFile = New cFileSM
        End If
        Set mcFileObj = mcLogFile
    End If
    
    If StringEmpty(mcFileObj.FileName) Then
        If blnError Then
            strFileName = gstrProjectPath & "\" & App.EXEName & ".ERR"
            strFileHdr = "Stepmaster Errors"
        Else
            strFileName = gstrProjectPath & "\" & App.EXEName & ".DBG"
            strFileHdr = "Stepmaster Log"
        End If
        
        mcFileObj.FileName = strFileName
        mcFileObj.WriteLine strFileHdr
        mcFileObj.WriteLine "Log start time : " & Now
    End If
    
    mcFileObj.WriteLine sMsg
    
    Exit Sub
    
WriteToFileErr:
    ' Display the error code raised by Visual Basic
    Call DisplayErrors(Errors)
    ' An error message would've been displayed by the called
    ' procedures
    
End Sub
Public Sub WriteMessage(sMsg As String)

    Call WriteToFile(sMsg, True)

End Sub

Sub BugTerm()
#If afDebug And afLogfile Then
    ' Close log file
    mcLogFile.CloseFile
#End If
End Sub

Public Sub ShowError(ByVal ErrorCode As errErrorConstants, _
        Optional ByVal ErrorSource As String = gstrEmptyString, _
        Optional ByVal OptArgs As String = gstrEmptyString, _
        Optional ByVal DoWriteError As Boolean = True)

    If DoWriteError Then
        ' Call a procedure to write the error to a log file
        Call WriteError(ErrorCode, ErrorSource, OptArgs)
    End If
    
    ' Re-initialize the values of the Error object before
    ' displaying the error to the user
    Call InitErrObject(ErrorCode, ErrorSource, OptArgs)
    
    Call DisplayErrors(Errors)
    
    Err.Clear
    
End Sub
Public Sub WriteError(ByVal ErrorCode As errErrorConstants, _
        Optional ByVal ErrorSource As String = gstrEmptyString, _
        Optional ByVal OptArgs As String = gstrEmptyString)

    ' Initialize the values of the Error object before
    ' calling the log function
    Call InitErrObject(ErrorCode, ErrorSource, OptArgs)
    
    Call LogErrors(Errors)
    
    Err.Clear
    
End Sub
Private Sub InitErrObject(ByVal ErrorCode As errErrorConstants, _
        Optional ByVal ErrorSource As String = gstrEmptyString, _
        Optional ByVal OptArgs As String = gstrEmptyString)
        
    Dim lngError As Long
    
    lngError = IIf(ErrorCode > vbObjectError And ErrorCode < vbObjectError + 65535, _
            ErrorCode - vbObjectError, ErrorCode)
    Err.Number = lngError + vbObjectError
    Err.Description = LoadResString(lngError) & OptArgs
    Err.Source = App.EXEName & ErrorSource
    
End Sub
Public Sub ShowMessage(ByVal MessageCode As errErrorConstants, _
        Optional ByVal OptArgs As String)

    Dim strMessage As String
    
    On Error GoTo ShowMessageErr
            
    strMessage = LoadResString(MessageCode) & OptArgs
    
    ' Write the error to a log file
    BugMessage strMessage
    
    MsgBox strMessage, vbOKOnly
    
    Exit Sub
        
ShowMessageErr:
    ' Log the error and exit
    Call DisplayErrors(Errors)
    
End Sub
Public Sub ShowMessageStr(sMessage As String)

    ' Write the error to a log file
    BugMessage sMessage
    
    MsgBox sMessage, vbOKOnly
    
End Sub

Public Sub DisplayErrors(myErrCollection As Errors)
    Dim strError As String
    Dim errLoop As Error
    Dim errCode As Long
    
    ' Enumerate Errors collection and display properties of
    ' each Error object.
    If Err.Number <> 0 Then
        If Err.Number > vbObjectError And Err.Number < (vbObjectError + 65536) Then
            errCode = Err.Number - vbObjectError
        Else
            errCode = Err.Number
        End If
        strError = "Error # " & Str(errCode) & " was generated by " _
            & Err.Source & Chr(13) & Err.Description
        MsgBox strError, , "Error", Err.HelpFile, Err.HelpContext
    Else
        For Each errLoop In myErrCollection
            With errLoop
                If Err.Number > vbObjectError And Err.Number < (vbObjectError + 65536) Then
                    errCode = .Number - vbObjectError
                Else
                    errCode = .Number
                End If
                strError = "Error #" & errCode & vbCrLf
                strError = strError & "    " & .Description & vbCrLf
                strError = strError & _
                    "    (Source: " & .Source & ")" & vbCrLf
'                strError = strError & _
'                    "Press F1 to see topic " & .HelpContext & vbCrLf
'                strError = strError & _
'                    "    in the file " & .HelpFile & "."
            End With
        
            MsgBox strError
        Next
    End If

End Sub
Public Sub LogErrors(myErrCollection As Errors)
    Dim cColErrors As cVectorStr
    Dim strError As String
    Dim errLoop As Error
    Dim errCode As Long
    Dim lngIndex As Long
    
    Set cColErrors = New cVectorStr
    
    ' Enumerate Errors collection and display properties of
    ' each Error object.
    If Err.Number <> 0 Then
        If Err.Number > vbObjectError And Err.Number < (vbObjectError + 65536) Then
            errCode = Err.Number - vbObjectError
        Else
            errCode = Err.Number
        End If
        strError = "Error # " & Str(errCode) & " was generated by " _
            & Err.Source & vbCrLf & Err.Description
        
        cColErrors.Add strError
    End If
    
    ' Log all database errors, if any
    For Each errLoop In myErrCollection
        With errLoop
            If Err.Number > vbObjectError And Err.Number < (vbObjectError + 65536) Then
                errCode = .Number - vbObjectError
            Else
                errCode = .Number
            End If
            strError = "Error #" & errCode & vbCrLf
            strError = strError & "    " & .Description & vbCrLf
            strError = strError & _
                "    (Source: " & .Source & ")" & vbCrLf
        End With
    
        cColErrors.Add strError
    Next
    
    ' We can have a error handler now that we have stored all
    ' errors away safely! - having an error handler before
    ' enumerating all the errors would have cleared the error
    ' collection
    On Error GoTo LogErrorsErr
    gstrSource = mstrModuleName & "LogErrors"

    For lngIndex = 0 To cColErrors.Count - 1
        strError = cColErrors(lngIndex)
        Debug.Print strError
        Call WriteToFile(strError, True)
    Next lngIndex
    
    Set cColErrors = Nothing
    
    Exit Sub
    
LogErrorsErr:
    ' Display the error code raised by Visual Basic
    DisplayErrors Errors
    On Error GoTo 0
    ShowError errUnableToWriteError, DoWriteError:=False
    
End Sub
